if (!window.LdapUserUtilsAjax) alert('AttributeScopeMenu requires LdapUserUtilsAjax.js');
if (!window.Layout) alert('AttributeScopeMenu requires Layout.js');
if (!window.Shim) alert('AttributeScopeMenu requires Shim.js');
if (!window.MessageFormat) alert('AttributeScopeMenu requires MessageFormat.js');

function AttributeScopeMenu(plainWidgetId, scopedWidgetId, labelId, linkId, scopesToAttributesMap, initialScope)
{
	this.label = labelId;
	this.link = linkId;
	this.scopesToAttributesMap = scopesToAttributesMap;
	this.plainWidgetId = plainWidgetId;
	this.scopedWidgetId = scopedWidgetId;
	this.initialScope = initialScope;

	this.hasScopes = false;
	for (var x in this.scopesToAttributesMap)
	{
		this.hasScopes = true;
		break;
	}
	
	this.defaultLabel = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=anyAttributeScopedLabel --><!-- /WSLET -->';
	this.lifecycleLabel = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=lifecycleXAttributeScopedLabel --><!-- /WSLET -->';

	// Use addOnLoad here so that it works on FF3 and others
	dojo.addOnLoad(dojo.hitch(this,'_init'));
}

AttributeScopeMenu.prototype.onChangeScope;
AttributeScopeMenu.prototype.onLoad;

AttributeScopeMenu.prototype.usePluralLabels = function()
{
	this.defaultLabel = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=allAttributesScopesLabel --><!-- /WSLET -->';
	this.lifecycleLabel = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=lifecycleXAttributesScopedLabel --><!-- /WSLET -->';
};

AttributeScopeMenu.prototype.show = function(e)
{
	if (e)
	{
		e.stopPropagation();
		e.preventDefault();
	}
	
	var pos = Layout.getBottomLeft(document.getElementById(this.link));
	Layout.positionInViewport(this.menu, pos.y, pos.x);
	Shim.shim(this.menu);

	this.showing = true;
};

AttributeScopeMenu.prototype.hide = function()
{
	if (!this.showing) return;

	Shim.removeShim(this.menu);
	dojo.style(this.menu, {display: "none"});
	document.getElementById(this.link).focus();

	this.showing = false;
};

AttributeScopeMenu.prototype.getScopeName = function(scope)
{
	return (typeof scope == 'undefined')
		? this.defaultLabel
		: new MessageFormat(this.lifecycleLabel).format([scope]);
};

AttributeScopeMenu.prototype._changeScope_onClick = function(e)
{
	var link = e.target;
	e.preventDefault();
	this.changeScope(link.scope);

	if ((typeof gMirroredMenuName != 'undefined') && link.scope && (link.scope.indexOf(gMirroredMenuName) == 0))
		return;

	LdapUserUtilsAjax.setStringPreference(window.token, 'attribute_scope', link.scope);
};
	
AttributeScopeMenu.prototype.changeScope = function(scope)
{
	if (!this.scopesToAttributesMap[scope])
	{
		scope = window.undefined;
	}

	replaceChildren(document.getElementById(this.label), document.createTextNode(this.getScopeName(scope)));

	this.lastScope = scope;
	if (this.onChangeScope) this.onChangeScope(scope);
};

function replaceChildren(node, newNode)
{
	while(node.hasChildNodes())
	{
		node.removeChild(node.firstChild);
	}
	node.appendChild(newNode);
}

AttributeScopeMenu.prototype.refreshScope = function()
{
	this.changeScope(this.lastScope);
};


AttributeScopeMenu.prototype._buildScopeRow = function(scope)
{
	var a = document.createElement('a');
	a.scope = scope;
	a.href = '#';
	a.appendChild(document.createTextNode(this.getScopeName(scope)));

	var td = document.createElement('td');
	td.scope = scope;
	td.appendChild(a);

	var tr = document.createElement('tr');
	tr.appendChild(td);
	
	dojo.connect(a, 'onclick', this, '_changeScope_onClick');

	// do our own highlighting to work around bugs with IE and a:hover
	dojo.connect(tr, 'onmouseover', function(){dojo.addClass(tr, 'highlight')});
	dojo.connect(tr, 'onmouseout', function(){dojo.removeClass(tr, 'highlight')});

	return tr;
};

AttributeScopeMenu.prototype._init = function()
{
	if (!this.hasScopes)
	{
		// still change to default scope to ensure we populate the list of attributes.
		this.changeScope();
		dojo.style(this.plainWidgetId, {display: ""});
		if (this.onLoad) this.onLoad();
		return;
	}
	
	dojo.connect(document.getElementById(this.link), 'onclick', this, 'show');
	dojo.connect(document, 'onclick', this, 'hide');

	// add the scopes to our menu
	this.menu = document.createElement('table');

	this.menu.className = 'menu-table';
	this.menu.style.display = 'none';
	
	var tbody = document.createElement('tbody');
	this.menu.appendChild(tbody);

	// default/all scope
	tbody.appendChild(
		this._buildScopeRow());

	for (var scope in this.scopesToAttributesMap)
	{
		tbody.appendChild(
			this._buildScopeRow(scope));
	}
	
	document.getElementById(this.scopedWidgetId).parentNode
		.appendChild(this.menu);

	this.changeScope(this.initialScope);
	dojo.style(this.scopedWidgetId, {display: ""});
	if (this.onLoad) this.onLoad();
};


