if (!window.DWRUtil) alert("SelectUtils.js requires dwr/util.js");

/**
 * Functions to interact with HTML SELECT objects.
 */
function SelectUtils()
{
}

/**
 * Adds the selected options from the "left" list to the "right" list.
 * Used when choosing from a list of available options and inserting into
 * a list of favorites.
 */
SelectUtils.copyOptions = function(fromList, toList)
{
	fromList = $(fromList);
	toList = $(toList);
	
	var existingOptions = new Object();

	// put all the favorite names into a hashtable for quick existence checks.
	// While at it, clear the selected toList so that it's easy to
	// see which new names are added.
	for (var i = 0; i < toList.options.length; ++i)
	{
		existingOptions[toList.options[i].text] = toList.options[i];
		toList.options[i].selected = false;
	}

	var selectedIndex = fromList.options.selectedIndex;
	
	if (-1 == selectedIndex)
	{
		return false;
	}
	
	var dirty;

	for (var i = fromList.options.selectedIndex; i < fromList.length; ++i)
	{
		var availableOption = fromList.options[i];

		if (availableOption.selected
			&& "" != availableOption.text)
		{
			var existing = existingOptions[availableOption.text];

			if (existing != null)
			{
				existing.selected = true;
			}
			else
			{
				dirty = true;

				var newOption = new Option(availableOption.text, availableOption.value);
				newOption.selected = true;
				toList.options[toList.options.length] = newOption;
			}
		}
	}

	return dirty;
};

/**
 * Removes all the selected options from a select.
 */
SelectUtils.removeSelected = function(select)
{
	select = $(select);
	var selectedIndex = select.selectedIndex;

	if (selectedIndex == -1) return;

	for (var i = select.options.length - 1; i >= selectedIndex; i--)
	{
		if (select.options[i].selected)
		{
			select.options[i] = null;
		}
	}
};

/**
 * Removes all the options with the given values
 * 
 * @select select to modify.
 * @values remove the options that have a value in this array.
 */
SelectUtils.removeValues = function(select, values)
{
	select = $(select);
	var valueSet = new Object();
	for (var i = 0; i < values.length; i++)
	{
		valueSet[values[i]] = true;
	}

	for (var i = select.options.length - 1; i >= 0; i--)
	{
		if (valueSet[select.options[i].value])
		{
			select.options[i] = null;
		}
	}
};

/**
 * Removes all options with the given text.
 */
SelectUtils.removeByText = function(select, text)
{
	select = $(select);
	
	for (var i = select.options.length - 1; i >= 0; i--)
	{
		if (select.options[i].text == text)
		{
			select.options[i] = null;
		}
	}
};

/**
 * Returns the values of all the selected options as an array.
 */
SelectUtils.getSelectedValues = function(select)
{
	select = $(select);
	var values = new Array();

	for (var i = 0; i < select.options.length; i++)
	{
		if (select[i].selected)
		{
			values.push(select[i].value);
		}
	}

	return values;
};

/**
 * Returns the text values of all the selected options as an array.
 */
SelectUtils.getSelectedTextValues = function(select)
{
	select = $(select);
	var values = new Array();

	for (var i = 0; i < select.options.length; i++)
	{
		if (select[i].selected)
		{
			values.push(select[i].text);
		}
	}

	return values;
};

SelectUtils.getValueSet = function(select)
{
	select = $(select);
	var values = new Object();

	for (var i = 0; i < select.options.length; i++)
	{
		values[select[i].value] = true;
	}

	return values;
};

SelectUtils.getValues = function(select)
{
	select = $(select);
	var values = new Array();

	for (var i = 0; i < select.options.length; i++)
	{
		values.push(select.options[i].value);
	}

	return values;
};

SelectUtils.getSelectedOption = function(select)
{
	select = $(select);
	var selectedIndex = select.selectedIndex;

	if (selectedIndex == -1) return null;

	return select.options[selectedIndex];
};

SelectUtils.getSelectedOptions = function(select)
{
	select = $(select);
	var selectedIndex = select.selectedIndex;
	var options = new Array();
	
	if (selectedIndex == -1) return options;
	
	for (var i = selectedIndex; i < select.options.length; i++)
	{
		var option = select.options[i];
		
		if (option.selected)
		{
			options.push(option);
		}
	}

	return options;
};

/**
 * Read a property off the (first) selected option.
 * 
 * @select the select to read.
 * @property the name of a property, e.g., 'text', 'value', etc.
 * @return the property value, or null if no option was selected
 *         or the property didn't exist.
 */
SelectUtils.getSelectedProperty = function(select, property)
{
	var option = SelectUtils.getSelectedOption(select);
	return option == null ? null : option[property];
};

SelectUtils.getSelectedValue = function(select)
{
	return SelectUtils.getSelectedProperty(select, 'value');
};

SelectUtils.getSelectedText = function(select)
{
	return SelectUtils.getSelectedProperty(select, 'text');
};

/**
 * Replaces the current options of a SELECT with a new list of options.
 * Useful if the same options are being set on a more than one select.
 * 
 * @select the select to modify.
 * @newOptions the new OPTIONs to put in the select.
 * @selectedValue if specified, the OPTION with this value will be selected.
 */
SelectUtils.setOptionCopies = function(select, newOptions, selectedValue)
{
	select = $(select);
	var selectedIndex = -1;
	select.options.length = 0;

	for (var i = 0; i < newOptions.length; i++)
	{
		var newOption = new Option(newOptions[i].text, newOptions[i].value);
		if (newOption.value == selectedValue)
		{
			selectedIndex = i;
		}

		select.options[select.options.length] = newOption;
	}
	
	select.selectedIndex = selectedIndex;
};

SelectUtils.setOptions = function(select, newOptions, selectedValue)
{
	select = $(select);
	var selectedIndex = -1;
	select.options.length = 0;

	for (var i = 0; i < newOptions.length; i++)
	{
		var newOption = newOptions[i];
		if (newOption.value == selectedValue)
		{
			selectedIndex = i;
		}

		select.options[select.options.length] = newOption;
	}
	
	select.selectedIndex = selectedIndex;
};

SelectUtils.setValues = function(select, values)
{
	select = $(select);
	select.options.length = 0;
	SelectUtils.addValues(select, values);
};

SelectUtils.addValue = function(select, value, setSelected)
{
	select = $(select);
	SelectUtils.addOption(select, value, value, setSelected);
};

SelectUtils.addOption = function(select, text, value, setSelected)
{
	select = $(select);
	var option = new Option(text, value, false, setSelected);
	select.options[select.length] = option;
	
	return option;
};

SelectUtils.addValues = function(select, values)
{
	for (var i = 0; i < values.length; i++)
	{
		SelectUtils.addValue(select, values[i]);
	}
};

SelectUtils.swapOptions = function(firstOption, secondOption)
{
	function setFirstOptionFromSecondOption(firstOption, secondOption)
	{
		firstOption.text = secondOption.text;
		firstOption.selected = secondOption.selected;
		firstOption.value = secondOption.value;
		firstOption.defaultSelected = secondOption.defaultSelected;
	}

	var tempOption = new Option();
	
	setFirstOptionFromSecondOption(tempOption, firstOption);
	setFirstOptionFromSecondOption(firstOption, secondOption);
	setFirstOptionFromSecondOption(secondOption, tempOption);
};

SelectUtils.moveSelectedUp = function(selectList)
{
	for (var i = selectList.selectedIndex; i < selectList.options.length; i++)
	{
		if (i <= 0)
		{
			// can't move up any farther
			continue;
		}
		
		var currentOption = selectList.options[i];
		
		if (!currentOption.selected)
		{
			continue;
		}

		var optionAbove = selectList.options[i - 1];
		
		if (optionAbove.selected)
		{
			// the option above is still selected, so it was moved up also.
			// Retain relative order.
			continue;
		}

		SelectUtils.swapOptions(optionAbove, currentOption);
	}
};

SelectUtils.moveSelectedDown = function(selectList)
{
	// move backwards through options
	for (var i = selectList.options.length - 1; i >= 0; --i)
	{
		if (i >= selectList.options.length - 1)
		{
			// can't move down any farther
			continue;
		}
		
		var currentOption = selectList.options[i];
		
		if (!currentOption.selected)
		{
			continue;
		}

		var optionBelow = selectList.options[i + 1];
		
		if (optionBelow.selected)
		{
			// the option below is still selected, so it was moved down also.
			// Retain relative order.
			continue;
		}

		SelectUtils.swapOptions(optionBelow, currentOption);
	}
};

/**
 * Selects a single OPTION in a SELECT.
 *
 * @select SELECT to modify.
 * @value the value of the OPTION to select.
 * @force if true and 'value' is not found, appends 'value' as a new OPTION.
 *		'value' is used as the text and value of the new OPTION.
 */
SelectUtils.selectValue = function(select, value, force)
{
	select = $(select);
	var options = select.options;

	for (var i = 0; i < select.options.length; i++)
	{
		var option = select.options[i];

		if (option.value == value)
		{ 
			option.selected = true;
			return true;
		}
	}
	
	// if we don't find the value, and 'force' is true, append the new value
	if (force)
	{
		options[select.options.length] = new Option(value, value, true);
	}
	
	return false;
};

SelectUtils.hasValue = function(select, value)
{
	select = $(select);
	for (var i = 0; i < select.options.length; i++)
	{
		var option = select.options[i];

		if (option.value == value)
		{
			return true;
		}
	}

	return false;
};

/**
 * Disables several OPTIONS at once in a SELECT. The 'disabled' property
 * is set, which works in Mozilla browsers; class 'disabled-option'
 * is added to the current class of the selected OPTIONs so they can be styled
 * --grayed out--in IE (which doesn't do anything with disabled on OPTIONs).
 * 
 * @select SELECT to modify.
 * @disabledValuesSet the values to disable as the property names of an Object. I.e.,
 *		a set of values. Note that all values other than these are enabled: 'disabled'
 *		is set to false.
 */
SelectUtils.disableValues = function(select, disabledValuesSet)
{
	select = $(select);
	for (var i = 0; i < select.options.length; i++)
	{
		var option = select.options[i];

		if (disabledValuesSet[option.value] && !option.selected)
		{
			option.disabled = true;
			if (window.StyleUtils) StyleUtils.addClass(option, 'disabled-option');
		}
		else
		{
			option.disabled = false;
			if (window.StyleUtils) StyleUtils.removeClass(option, 'disabled-option');
		}
	}
};
